"""
SCout | Special Character Out

Copyright 2017-2019, Marcel Schmalzl (MSc)
This code is licensed under a MIT license
You should have received a copy of the MIT licence along with this program.
If not see:
* https://github.com/holzkohlengrill/SCout/blob/master/LICENSE
* https://opensource.org/licenses/MIT
"""

import sys


class Singleton(type):
    """
    Singleton metaclass
    """
    __instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls.__instances:
            cls.__instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        # FIXME: It would be prettier if we can change the settings via `sc(<params>)`
        #       the following code would allow this however we would get a change to default values for a call like `sc().header("test")`
        #       `chgSettings()` would then do the same like the current implementation of `Logger.__call__(self, invVerbosity: int = 0, actionWarning=None, actionError=None)`
        # else:
        #     cls.__instances[cls].chgSettings(*args, **kwargs)
        return cls.__instances[cls]


def _canUseColour():
    """
    Checks if coloured text output is supported
    :return: True if colours are supported, otherwise False
    """
    supportedPlatforms = ["linux", "cygwin", "darwin"]
    platformSupported = False

    for platform in supportedPlatforms:
        if sys.platform.startswith(platform):
            platformSupported = True
            break

    isTty = hasattr(sys.stdout, 'isatty') and not sys.stdout.isatty()
    if platformSupported or isTty:
        return True
    else:
        return False


class Colours:
    """
    Colour definitions
    """
    WHITE = '\033[37m'          # Indeed very light grey
    LIGHT_CYAN = '\033[36m'
    BLUE = '\033[94m'
    GREEN = '\033[92m'
    WARNING = '\033[93m'        # Yellow
    WWARNING = '\033[33m'       # Dark Grey
    ERROR = '\033[91m'          # Light red
    HEADER = '\033[1m'          # Bold white
    UNDERLINE = '\033[4m'

    NO_COLOUR = '\033[0m'


def _checkNconvertStr(texts):
    """
    Checks whether input is a string or if it can be casted into one
    :param texts: A string or a type which can be converted into one
    :return: Text as a string if successful otherwise nothing (exception)
    """
    concatTextLst = []
    for text in texts:
        # Test texts elements
        tempText = str(text)
        if type(tempText) is not str:
            raise TypeError("Input type must be a string or convertible into a string")
        concatTextLst.append(str(text))

    return " ".join(concatTextLst)      # Preserve whitespaces when using multiple text elements



"""
SCout | Special Character Out

Copyright 2017-2019, Marcel Schmalzl (MSc)
This code is licensed under a MIT license
You should have received a copy of the MIT licence along with this program.
If not see:
* https://github.com/holzkohlengrill/SCout/blob/master/LICENSE
* https://opensource.org/licenses/MIT
"""

__all__ = ['SCout', 'SCout_Logger']
from pypiscout.SCout import debug, info, wwarning, warning, error, header
from pypiscout.SCout_Logger import Logger



"""
SCout | Special Character Out

Copyright 2017-2019, Marcel Schmalzl (MSc)
This code is licensed under a MIT license
You should have received a copy of the MIT licence along with this program.
If not see:
* https://github.com/holzkohlengrill/SCout/blob/master/LICENSE
* https://opensource.org/licenses/MIT
"""

from pypiscout.SCout_helpers import _canUseColour, _checkNconvertStr, Colours

__ALIGNMENT = 16


class Header:
    """
    Generate header
    """
    @staticmethod
    def _genTopBottom(text, symbol):
        """
        Generate the top or bottom of the header
        :param text: Text
        :param symbol: Symbol
        :return: String for top + bottom header
        """
        neededLength = len(text) + 8  # Two spaces + 3 x 2 slashes
        return str(symbol * neededLength)

    @staticmethod
    def _genMiddle(text, symbol):
        """
        Generate the middle of the header
        :param text: Text
        :param symbol: Symbol
        :return: String for middle header
        """
        # TODO: Add multi-line support (MSc)
        # Workaround:
        if "\n" in "".join(text):
            for txt in text:
                txt.replace("\n", "; ")
            print(text)

        return " ".join([symbol * 3, text, symbol * 3])  # 3 synbols + text + 3 symbols separated by spaces

    @staticmethod
    def gen(text, symbol="/"):
        """
        Generate a header
        :param text: Text
        :param symbol: Symbol (default: "/")
        :return: Header as string
        """
        return Header._genTopBottom(text, symbol) + "\n" + Header._genMiddle(text, symbol) + "\n" + Header._genTopBottom(text, symbol) + "\n"


def debug(*text, disableColour=False):
    """
    Prints a debug message (usable like normal print)
    :param disableColour: disable coloured output (default: False)
    :param text: See doc of `_checkNconvertStr`
    :return: The text printed
    """
    colour = ""
    stdColour = ""
    if _canUseColour() and not disableColour:
        colour = Colours.LIGHT_CYAN
        stdColour = Colours.NO_COLOUR
    textOut = " ".join([colour + '(debug) :'.rjust(__ALIGNMENT), _checkNconvertStr(text) + stdColour])
    print(textOut)
    return textOut


def info(*text, disableColour=False):
    """
    Prints a info message (usable like normal print)
    :param disableColour: disable coloured output (default: False)
    :param text: See doc of `_checkNconvertStr`
    :return: The text printed
    """
    colour = ""
    stdColour = ""
    if _canUseColour() and not disableColour:
        colour = Colours.WHITE
        stdColour = Colours.NO_COLOUR
    textOut = " ".join([colour + ' (info) :'.rjust(__ALIGNMENT), _checkNconvertStr(text) + stdColour])
    print(textOut)
    return textOut


def wwarning(*text, disableColour=False):
    """
    Prints a weak warning message (usable like normal print)
    :param disableColour: disable coloured output (default: False)
    :param text: See doc of `_checkNconvertStr`
    :return: The text printed
    """
    colour = ""
    stdColour = ""
    if _canUseColour() and not disableColour:
        colour = Colours.WWARNING
        stdColour = Colours.NO_COLOUR
    textOut = " ".join([colour + '(weak warning) :'.rjust(__ALIGNMENT), _checkNconvertStr(text) + stdColour])
    print(textOut)
    return textOut


def warning(*text, disableColour=False):
    """
    Prints a warning message (usable like normal print)
    :param disableColour: disable coloured output (default: False)
    :param text: See doc of `_checkNconvertStr`
    :return: The text printed
    """
    colour = ""
    stdColour = ""
    if _canUseColour() and not disableColour:
        colour = Colours.WARNING
        stdColour = Colours.NO_COLOUR
    textOut = " ".join([colour + '(warning) :'.rjust(__ALIGNMENT), _checkNconvertStr(text) + stdColour])
    print(textOut)
    return textOut


def error(*text, disableColour=False):
    """
    Prints an error message (usable like normal print)
    :param disableColour: disable coloured output (default: False)
    :param text: See doc of `_checkNconvertStr`
    :return: The text printed
    """
    colour = ""
    stdColour = ""
    if _canUseColour() and not disableColour:
        colour = Colours.ERROR
        stdColour = Colours.NO_COLOUR
    textOut = " ".join([colour + '(error) :'.rjust(__ALIGNMENT), _checkNconvertStr(text) + stdColour])
    print(textOut)
    return textOut


def custom(*text, disableColour=False, colourCode=Colours.WHITE, prefix='info', alignment=__ALIGNMENT):
    """
    Prints a customised message (usable like normal print)
    :param text:  See doc of `_checkNconvertStr`
    :param disableColour: disable coloured output (default: False)
    :param colourCode: custom colour (int of foreground colour ANSI/VT100 Control sequence) (default: white)
    :param prefix: Text to display like `warning`, `error`, `info`, ... (default: info)
    :param alignment: Padding of text vs. prefix
    :return: The text printed
    """
    colour = ""
    stdColour = ""
    if _canUseColour() and not disableColour:
        colour = '\033[' + colourCode + 'm'
        stdColour = Colours.NO_COLOUR
    textOut = " ".join([colour + '(' + str(prefix) + ') :'.rjust(alignment), _checkNconvertStr(text) + stdColour])
    print(textOut)
    return textOut


def header(*text, symbol="/", disableColour=False):
    """
    Prints a header in the following style
    :param disableColour: disable coloured output (default: False)
    :param text: Header text
    :param symbol: Symbol used for header (default: "/")
    :return: The text printed
    """
    colour = ""
    stdColour = ""
    if _canUseColour() and not disableColour:
        colour = Colours.BLUE
        stdColour = Colours.NO_COLOUR
    textOut = " ".join([colour + Header.gen(_checkNconvertStr(text), symbol=symbol) + stdColour])
    print(textOut)
    return textOut



"""
SCout | Special Character Out

Copyright 2017-2019, Marcel Schmalzl (MSc)
This code is licensed under a MIT license
You should have received a copy of the MIT licence along with this program.
If not see:
* https://github.com/holzkohlengrill/SCout/blob/master/LICENSE
* https://opensource.org/licenses/MIT
"""

from pypiscout.SCout import header, debug, info, wwarning, warning, error
from pypiscout.SCout_helpers import Singleton

_VERSION_MAJOR = "2"
_VERSION_MINOR = "0"
VERSION = ".".join([_VERSION_MAJOR, _VERSION_MINOR])


class Logger(metaclass=Singleton):
    """
    SCout logging class
    """
    class Settings:
        """
        Initialises generic settings:
         * invVerbosity level
         * actions if an error or warning occurred (provide a function to set behaviour)
        """
        def __init__(self, invVerbosity: int, actionWarning, actionError):
            """
            Initialise settings
            Arguments: see Logger.__init__ docstring
            """
            self.invVerbosity = invVerbosity
            self.actionWarning = actionWarning
            self.actionError = actionError

    def __init__(self, invVerbosity: int = 0, actionWarning=None, actionError=None):
        """
        Set settings
        :param invVerbosity: Inverse invVerbosity levels: (default: 0)
                        -1  : Print all (debug, info, weak warnings, warning & error)
                        0   : Print info, weak warnings, warning & error
                        1   : Print weak warnings, warning & error
                        2   : Print warning & error
                        3   : Print error
                        4   : Do NOT print anything
        :param actionWarning: Provide a function which is executed in case of a warning (default: None)
        :param actionError: Provide a function which is executed in case of an error (default: None)
        """
        self.__settings = Logger.Settings(invVerbosity, actionWarning, actionError)

    def __call__(self, invVerbosity: int = 0, actionWarning=None, actionError=None):
        """
        Exactly like init
        This callable let the user change the settings later on
        """
        self.__settings = Logger.Settings(invVerbosity, actionWarning, actionError)

    def debug(self, *text, disableColour=False):
        if 1 > self.__settings.invVerbosity:
            return debug(*text, disableColour=disableColour)

    def info(self, *text, disableColour=False):
        """
        Info (general information to be printed)
        :param disableColour: disable coloured output (default: False)
        :param text: See doc of `_checkNconvertStr`
        :return: The text printed
        """
        if 2 > self.__settings.invVerbosity:
            return info(*text, disableColour=disableColour)

    def warning(self, *text, disableColour=False):
        """
        Warning
        :param disableColour: disable coloured output (default: False)
        :param text: See doc of `_checkNconvertStr`
        :return: The text printed
        """
        if 3 > self.__settings.invVerbosity:
            return warning(*text, disableColour=disableColour)
        if self.__settings.actionWarning:
            self.__settings.actionWarning()

    def wwarning(self, *text, disableColour=False):
        """
        Weak warning (like warning but less prominent)
        Does not have an associated warning action
        :param disableColour: disable coloured output (default: False)
        :param text: See doc of `_checkNconvertStr`
        :return: The text printed
        """
        if 3 > self.__settings.invVerbosity:
            return wwarning(*text, disableColour=disableColour)

    def error(self, *text, disableColour=False):
        """
        Error message (something prevents the application from continuing)
        :param disableColour: disable coloured output (default: False)
        :param text: See doc of `_checkNconvertStr`
        :return: The text printed
        """
        if 4 > self.__settings.invVerbosity:
            error(*text, disableColour=disableColour)
        if self.__settings.actionError:
            self.__settings.actionError()

    def header(self, *text, symbol="/", disableColour=False):
        """
        Header (title alike)
        :return: The text printed
        :param text: See doc of `_checkNconvertStr`
        :param symbol: Character decorating the header (default: "/")
        :param disableColour: disable coloured output (default: False)
        :return: The text printed
        """
        if 5 > self.__settings.invVerbosity:
            return header(*text, symbol=symbol, disableColour=disableColour)



import setuptools

with open("readme.md", "r") as fp:
    long_description = fp.read()

setuptools.setup(
    name="pypiscout",
    version=2.0,
    license="MIT",
    author="Marcel Schmalzl",
    description="Prints formatted console output tagged as info, warning, error, etc. Usage is similar to `print()`.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/holzkohlengrill/SCout",
    packages=["pypiscout"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
